﻿package 
{
	//*****************************************************************************
	// import the required libraries (including the Papervision3D ones)
	//*****************************************************************************
	import flash.display.Bitmap;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import org.papervision3d.cameras.Camera3D;
	import org.papervision3d.materials.BitmapAssetMaterial;
	import org.papervision3d.materials.MaterialsList;
	import org.papervision3d.objects.DisplayObject3D;
	import org.papervision3d.scenes.Scene3D;
	import org.papervision3d.objects.Collada;
		
	public class XSIModTool extends Sprite 
	{ 
		private var m_Materials:Object;
		private var m_Container:Sprite;
		private var m_Scene:Scene3D;
		private var m_Camera:Camera3D;
		private var m_RootNode:DisplayObject3D;
		private var m_Gear1:DisplayObject3D;
		private var m_Gear2:DisplayObject3D;
		private var m_Gear3:DisplayObject3D;
		private var m_Letters:DisplayObject3D;
		private var m_PreviousMouseX:int;
		private var m_PreviousMouseY:int;
		private var m_MouseDown:Boolean;
		
		public function XSIModTool()
		{
			//*****************************************************************************
			// Material creation
			//
			// The name of the material here must match the name of the material assigned
			// to the 3D objects in the XSI Mod Tool.
			//*****************************************************************************
			m_Materials = 
			{
				XSIModTool_Texture: new BitmapAssetMaterial( "XSIModTool_Texture" )
			};
				
			//*****************************************************************************
			// Initialization of the 3D scene
			//*****************************************************************************
			Initialize3DScene();
				
			//*****************************************************************************
			// Add an event at the begining of each frame for animation and interactivity
			//*****************************************************************************
			addEventListener(Event.ENTER_FRAME, EnterFrameEvent);

			//*****************************************************************************
			// Trap the mouse down and up event both on the stage and on the 3D scene 
			// objects.
			//*****************************************************************************
			addEventListener(MouseEvent.MOUSE_DOWN, MouseDownEvent);
			addEventListener(MouseEvent.MOUSE_UP, MouseUpEvent);
			stage.addEventListener(MouseEvent.MOUSE_DOWN, MouseDownEvent);
			stage.addEventListener(MouseEvent.MOUSE_UP, MouseUpEvent);
		}
			
		//*****************************************************************************
		// Creates the container, scene, camera and root node
		//*****************************************************************************
		private function Initialize3DScene():void 
		{
			//*****************************************************************************
			// create the container, add it to the stage, position it 
			//*****************************************************************************
			m_Container = new Sprite();
			addChild( m_Container );
			m_Container.x = 200;
			m_Container.y = 200; 
						
			//*****************************************************************************
			// create a new scene and use the container 
			//*****************************************************************************
			m_Scene = new Scene3D( m_Container );

			m_MouseDown = false;
				
			//*****************************************************************************
			// create a new camera and position it 
			//*****************************************************************************
			m_Camera = new Camera3D(); 
			m_Camera.x = 3000;
			m_Camera.z = -30;
			m_Camera.zoom = 19; 
			m_Camera.focus = 30; 
				
			//*****************************************************************************
			// add a root node to the scene and add our 3d object by
			// reading the Collada (*.dae) file 
			//*****************************************************************************
			m_RootNode = m_Scene.addChild( new DisplayObject3D("rootNode") );

			//*****************************************************************************
			// TEXT: mod tOol
			//*****************************************************************************
			m_Gear1 = m_RootNode.addChild( new Collada( "gear.dae", new MaterialsList( m_Materials ) ) );
			m_Gear1.rotationZ = 90;
			m_Gear1.x = 0;
			m_Gear1.y = 40;
			m_Gear1.z = -155;
			
			m_Gear1.rotationY = 0;

			//*****************************************************************************
			// TEXT: mOd tool
			//*****************************************************************************
			m_Gear2 = m_RootNode.addChild( new Collada( "gear.dae", new MaterialsList( m_Materials ) ) );
			m_Gear2.rotationZ = 90;
			m_Gear2.x = 0;
			m_Gear2.y = 520;
			m_Gear2.z = 70;
			
			m_Gear2.rotationY = 18;

			//*****************************************************************************
			// TEXT: mod toOl
			//*****************************************************************************
			m_Gear3 = m_RootNode.addChild( new Collada( "gear.dae", new MaterialsList( m_Materials ) ) );			
			m_Gear3.rotationZ = 90;
			m_Gear3.x = 0;
			m_Gear3.y = 160;
			m_Gear3.z = 470;

			m_Gear3.rotationY = 26;

			//*****************************************************************************
			// TEXT: MoD TooL
			//*****************************************************************************
			m_Letters = m_RootNode.addChild( new Collada( "letters.dae", new MaterialsList( m_Materials ) ) );
			m_Letters.rotationZ = -90;
			m_Letters.rotationY = -90;
			m_Letters.x = -50;
			m_Letters.y = 195;
			m_Letters.z = 140;
			
			m_RootNode.y = -120;
			m_RootNode.z = 100;
			
			m_PreviousMouseX = m_Container.mouseX;
			m_PreviousMouseY = m_Container.mouseY;
		}
			
		//*****************************************************************************
		// The animation loop
		//*****************************************************************************
		private function EnterFrameEvent( event:Event ):void 
		{
			//*****************************************************************************
			// Mouse interaction
			//*****************************************************************************
			var l_IncrementX:int = (m_Container.mouseX - m_PreviousMouseX) / 4;
			m_PreviousMouseX = m_Container.mouseX;
			
			if (l_IncrementX > 10)
			{
				l_IncrementX = 10;
			}
			if (l_IncrementX < -10)
			{
				l_IncrementX = -10;
			}

			var l_IncrementY:int = (m_Container.mouseY - m_PreviousMouseY) / 4;
			m_PreviousMouseY = m_Container.mouseY;
			
			if (l_IncrementY > 10)
			{
				l_IncrementY = 10;
			}
			if (l_IncrementY < -10)
			{
				l_IncrementY = -10;
			}

			if (m_MouseDown)
			{
				m_RootNode.rotationY -= l_IncrementX;
				m_RootNode.rotationZ -= l_IncrementY;
			}

			//*****************************************************************************
			// Engrenage rotation animation
			//*****************************************************************************
			m_Gear1.rotationY -= 2;
			m_Gear2.rotationY += 2;
			m_Gear3.rotationY -= 2;
				
			//*****************************************************************************
			// render the camera to see the changes 
			//*****************************************************************************
			m_Scene.renderCamera( m_Camera );
		}
		
		public function MouseDownEvent(event:MouseEvent):void 
		{
			m_MouseDown = true;
		}		

		public function MouseUpEvent(event:MouseEvent):void 
		{
			m_MouseDown = false;
		}		
	}
}